<?php /**
 * @file
 * Contains \Drupal\ms_affiliates\EventSubscriber\InitSubscriber.
 */

namespace Drupal\ms_affiliates\EventSubscriber;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class InitSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [KernelEvents::REQUEST => ['onEvent', 0]];
  }

  public function onEvent() {
    $user = \Drupal::currentUser();
    if (!$user->uid AND (isset($_REQUEST['an']) OR isset($_REQUEST['a']))) {
      $account = NULL;
      if (isset($_REQUEST['a'])) {
        $account = user_load($_REQUEST['a']);
      }
      elseif (isset($_REQUEST['an'])) {
        $account = user_load_by_name(urldecode($_REQUEST['an']));
      }

      if (!empty($account->uid)) {
        // Set the session variable.
        $_SESSION['affiliate'] = $account->uid;
        $_SESSION['af_name'] = $account->name;

        // Get the redirect.
        $goto = drupal_get_path_alias($_GET['q']);
        if (empty($goto)) {
          $goto = variable_get('ms_affiliates_af_redirect', '');
        }

        // Set the cookie.
        ms_affiliates_set_cookie($_SESSION['affiliate']);

        // Record the Click.
        ms_affiliates_clicked($account, $goto);

        // Redirect to clear the '?a=#'.
        drupal_goto($goto, [
          'query' => ms_affiliates_remove_GET_parameter()
          ]);
      }
    }

    // Load the affiliate from the order, used in PayPal WPS return page.
    if (isset($_POST['custom'])) {
      if ($order = ms_core_order_load($_POST['custom']) AND isset($order->data['affiliate_aid']) AND $order->data['affiliate_aid']) {
        $_SESSION['affiliate'] = $order->data['affiliate_aid'];
      }
    } // Repopulate from Cookie.
    elseif (isset($_COOKIE['affiliate']) AND !isset($_SESSION['affiliate'])) {
      $_SESSION['affiliate'] = isset($_COOKIE['affiliate']['uid']) ? $_COOKIE['affiliate']['uid'] : 0;
      $_SESSION['af_name'] = isset($_COOKIE['affiliate']['name']) ? $_COOKIE['affiliate']['name'] : '';
      $_SESSION['affiliate_cookie_timestamp'] = isset($_COOKIE['affiliate']['timestamp']) ? $_COOKIE['affiliate']['timestamp'] : time();
    }
  }

}
