<?php /**
 * @file
 * Contains \Drupal\ms_coupons\Controller\DefaultController.
 */

namespace Drupal\ms_coupons\Controller;

use Drupal\Core\Controller\ControllerBase;

/**
 * Default controller for the ms_coupons module.
 */
class DefaultController extends ControllerBase {

  public function ms_coupons_display($view_type = 'active') {
    $header[] = [
      'data' => t('Name'),
      'field' => 'name',
    ];
    $header[] = [
      'data' => t('Code'),
      'field' => 'code',
      'sort' => 'asc',
    ];
    $header[] = [
      'data' => t('Value'),
      'field' => 'value',
    ];
    $header[] = [
      'data' => t('Scope'),
      'field' => 'scope',
    ];
    $header[] = [
      'data' => t('Valid until'),
      'field' => 'valid_until',
    ];
    $header[] = [
      'data' => t('Usage'),
      'field' => 'used',
    ];
    $header[] = ['data' => t('Actions')];

    $query = db_select('ms_coupons', 'c');
    $query->fields('c', [
      'cid',
      'name',
      'value',
      'used',
      'scope',
      'code',
      'type',
      'valid_until',
    ]);
    $query->condition('c.status', $view_type == 'inactive' ? 0 : 1, '=');

    $query = $query->extend('TableSort')->orderByHeader($header);
    $query = $query->extend('PagerDefault')->limit(20);
    $result = $query->execute();

    $rows = [];
    foreach ($result as $row) {
      if ($row->type == 'percentage') {
        $value = $row->value . '%';
      }
      else {
        $value = ms_core_format_money($row->value);
      }

      $code = $row->code;
      // @FIXME
      // l() expects a Url object, created from a route name or external URI.
      // $actions = l(t('edit'), "admin/moneysuite/ms_coupons/edit/" . $row->cid);


      // @FIXME
      // l() expects a Url object, created from a route name or external URI.
      // $actions .= ' ' . l(t('delete'), "admin/moneysuite/ms_coupons/delete/" . $row->cid);


      $rows[] = [
        $row->name,
        $code,
        $value,
        ms_core_get_scope_name($row->scope),
        format_date($row->valid_until, 'small'),
        $row->used,
        $actions,
      ];
    }

    if (count($rows)) {
      // @FIXME
// theme() has been renamed to _theme() and should NEVER be called directly.
// Calling _theme() directly can alter the expected output and potentially
// introduce security issues (see https://www.drupal.org/node/2195739). You
// should use renderable arrays instead.
// 
// 
// @see https://www.drupal.org/node/2195739
// $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('width' => '100%'), 'empty' => t('No coupons.')));

      // @FIXME
// theme() has been renamed to _theme() and should NEVER be called directly.
// Calling _theme() directly can alter the expected output and potentially
// introduce security issues (see https://www.drupal.org/node/2195739). You
// should use renderable arrays instead.
// 
// 
// @see https://www.drupal.org/node/2195739
// $output .= theme('pager', array('tags' => NULL));

    }
    else {
      $output = '<p>' . t('There are currently no !type coupons in the system.', [
        '!type' => $view_type
        ]) . '</p>';
    }

    return $output;
  }

}
