<?php

/**
 * @file
 * Contains \Drupal\ms_coupons\Form\MsCouponsAddForm.
 */

namespace Drupal\ms_coupons\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class MsCouponsAddForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ms_coupons_add_form';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state, $action = NULL, $cid = NULL) {
    if ($action == 'edit') {
      $coupon = ms_coupons_load($cid);
      $value = $coupon;
      $form['#ms_coupons_cid'] = $value->cid;
      $form['#ms_coupons'] = $value;
      $used = $coupon->used;
    }
    else {
      $value = new stdClass();
      $value->valid_until = REQUEST_TIME;
      $value->minimum_order = 0;
      $value->max_uses = 0;
      $value->name = '';
      $value->code = '';
      $value->scope = 'recurring';
      $value->status = '';
      $value->type = '';
      $value->minimum_order = '';
      $value->value = '';
      $used = 0;
    }

    $value->valid_until = [
      'year' => format_date($value->valid_until, 'custom', 'Y'),
      'month' => format_date($value->valid_until, 'custom', 'n'),
      'day' => format_date($value->valid_until, 'custom', 'j'),
    ];

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => t('Coupon name'),
      '#default_value' => $value->name,
      '#required' => TRUE,
    ];

    $form['code'] = [
      '#type' => 'textfield',
      '#title' => t('Coupon code'),
      '#description' => t('Coupon codes cannot be changed once they have been used in an order.'),
      '#default_value' => $value->code,
      '#size' => 25,
      '#required' => !$used,
      '#maxlength' => 14,
    ];
    if ($value->code AND $used) {
      $form['code']['#attributes'] = ['readonly' => 'readonly'];
    }

    $form['valid_until'] = [
      '#type' => 'date',
      '#title' => t('Expiry date'),
      '#default_value' => $value->valid_until,
      '#required' => TRUE,
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => t('Active'),
      '#description' => t('Check to enable the coupon, uncheck to disable the coupon.'),
      '#default_value' => $value->status,
    ];

    $form['type'] = [
      '#type' => 'select',
      '#title' => t('Discount type'),
      '#default_value' => $value->type,
      '#options' => ms_coupons_get_types(),
    ];

    $form['scope'] = [
      '#type' => 'radios',
      '#title' => t('Scope'),
      '#required' => TRUE,
      '#default_value' => $value->scope,
      '#options' => ms_core_get_scopes(),
      '#description' => t('Whether the coupon should affect the first payment only or all payments. Only applies for Recurring Payments.'),
    ];

    $form['value'] = [
      '#type' => 'textfield',
      '#title' => t('Discount value'),
      '#default_value' => $value->value,
      '#size' => 10,
      '#description' => t('Enter values without symbols, for 15%, enter "15" and choose Percentage as the discount type.'),
      '#required' => TRUE,
    ];

    $form['minimum_order'] = [
      '#type' => 'textfield',
      '#title' => t('Minimum order limit'),
      '#default_value' => $value->minimum_order,
      '#size' => 10,
      '#description' => t('A minimum order total that applies to the coupon, or 0 for no minimum order limit.'),
      '#required' => TRUE,
    ];

    $form['max_uses'] = [
      '#type' => 'textfield',
      '#title' => t('Maximum number of redemptions (per code)'),
      '#default_value' => $value->max_uses,
      '#description' => t('Enter the maximum number of times each code for this coupon can be used, or 0 for unlimited.'),
      '#size' => 5,
      '#required' => TRUE,
    ];

    $form['max_uses_per_user'] = [
      '#type' => 'textfield',
      '#title' => t('Maximum number of redemptions (per user)'),
      '#default_value' => isset($value->data['max_uses_per_user']) ? $value->data['max_uses_per_user'] : 0,
      '#description' => t('Enter the maximum number of times this coupon can be used by a single user, or 0 for unlimited.'),
      '#size' => 5,
      '#required' => TRUE,
    ];

    $form['users'] = [
      '#type' => 'fieldset',
      '#title' => t('User restrictions'),
      '#description' => t('Enter one or more user names and/or "anonymous users" below to make this coupon valid only for those users.'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => !isset($value->data['users']),
    ];

    if (isset($value->data['users'])) {
      foreach ($value->data['users'] as $username) {
        $form['users'][] = [
          '#type' => 'textfield',
          '#default_value' => $username,
          '#autocomplete_path' => 'ms_core/autocomplete/user',
        ];
      }
    }

    for ($i = 0; $i < 3; $i++) {
      $form['users'][] = [
        '#type' => 'textfield',
        '#autocomplete_path' => 'ms_core/autocomplete/user',
      ];
    }

    $form['roles'] = [
      '#type' => 'fieldset',
      '#title' => t('Role restrictions'),
      '#description' => t('Enter one or more role names below to make this coupon valid only for users with those roles.'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => !isset($value->data['roles']),
    ];

    if (isset($value->data['roles'])) {
      foreach ($value->data['roles'] as $role) {
        $form['roles'][] = [
          '#type' => 'textfield',
          '#default_value' => $role,
          '#autocomplete_path' => 'ms_core/autocomplete/role',
        ];
      }
    }

    for ($i = 0; $i < 3; $i++) {
      $form['roles'][] = [
        '#type' => 'textfield',
        '#autocomplete_path' => 'ms_core/autocomplete/role',
      ];
    }

    $form['products'] = [
      '#type' => 'fieldset',
      '#title' => t('Product restrictions'),
      '#description' => t('Enter one or more products below to make this coupon valid only for those products. Just enter the product name and it will be auto-completed.'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => !isset($value->data['products']),
    ];
    if (isset($value->data['products'])) {
      foreach ($value->data['products'] as $id) {
        $form['products'][] = [
          '#type' => 'textfield',
          '#default_value' => $id,
          '#autocomplete_path' => 'ms_core/autocomplete/product',
        ];
      }
    }
    for ($i = 0; $i < 3; $i++) {
      $form['products'][] = [
        '#type' => 'textfield',
        '#autocomplete_path' => 'ms_core/autocomplete/product',
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Submit'),
    ];

    return $form;
  }

  public function validateForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    // Ensure a unique coupon code.
    if ($coupon = ms_coupons_find($form_state->getValue([
      'code'
      ]))) {
      if ($coupon->cid != $form['#ms_coupons_cid']) {
        $form_state->setErrorByName('code', t('Coupon code already used by %name.', [
          '%name' => $coupon->name
          ]));
      }
    }
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $code = strtoupper($form_state->getValue(['code']));
    $valid_until = mktime(0, 0, 0, $form_state->getValue(['valid_until', 'month']), $form_state->getValue(['valid_until', 'day']), $form_state->getValue(['valid_until', 'year']));
    $data = [];

    if ($form_state->getValue(['max_uses_per_user'])) {
      $data['max_uses_per_user'] = $form_state->getValue(['max_uses_per_user']);
    }

    foreach ($form_state->getValue(['users']) as $username) {
      if (trim($username)) {
        $data['users'][] = $username;
      }
    }

    foreach ($form_state->getValue(['roles']) as $role) {
      if ($role) {
        $data['roles'][] = $role;
      }
    }

    foreach ($form_state->getValue(['products']) as $id) {
      if ($id) {
        $data['products'][] = $id;
      }
    }

    if (!isset($form['#ms_coupons_cid'])) {
      // Only set bulk coupon seed once.
      db_insert('ms_coupons')
        ->fields([
        'name' => $form_state->getValue(['name']),
        'code' => $code,
        'value' => floatval($form_state->getValue(['value'])),
        'type' => $form_state->getValue(['type']),
        'scope' => $form_state->getValue(['scope']),
        'status' => $form_state->getValue(['status']),
        'valid_until' => $valid_until,
        'max_uses' => $form_state->getValue(['max_uses']),
        'minimum_order' => $form_state->getValue(['minimum_order']),
        'data' => serialize($data),
      ])
        ->execute();
      drupal_set_message(t('Coupon %name has been created.', [
        '%name' => $form_state->getValue(['name'])
        ]));
    }
    else {
      db_update('ms_coupons')
        ->fields([
        'name' => $form_state->getValue(['name']),
        'code' => $code,
        'value' => floatval($form_state->getValue(['value'])),
        'type' => $form_state->getValue(['type']),
        'scope' => $form_state->getValue(['scope']),
        'status' => $form_state->getValue(['status']),
        'valid_until' => $valid_until,
        'max_uses' => $form_state->getValue(['max_uses']),
        'minimum_order' => $form_state->getValue(['minimum_order']),
        'data' => serialize($data),
      ])
        ->condition('cid', $form['#ms_coupons_cid'])
        ->execute();
      drupal_set_message(t('Coupon %name has been updated.', [
        '%name' => $form_state->getValue(['name'])
        ]));
    }

    $form_state->set(['redirect'], 'admin/moneysuite/ms_coupons' . ($form_state->getValue(['status']) ? '' : '/inactive'));
  }

}
