<?php

/**
 * @file
 * Contains \Drupal\ms_discounts\Form\MsDiscountsAddForm.
 */

namespace Drupal\ms_discounts\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class MsDiscountsAddForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ms_discounts_add_form';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state, $action = NULL, $id = NULL) {
    if ($action == 'edit') {
      $discount = ms_discounts_load($id);
      $form['#ms_discounts_id'] = $discount->id;
      $form['#ms_discounts'] = $discount;
    }
    else {
      $discount = new stdClass();
      $discount->valid_until = REQUEST_TIME;
      $discount->max_uses = 0;
      $discount->name = '';
      $discount->scope = 'recurring';
      $discount->status = '';
      $discount->type = '';
      $discount->minimum_order = '';
      $discount->value = '';
      $discount->data = [
        'minimum_order' => 0,
        'roles' => [],
        'products' => [],
        'users' => [],
        'max_per_user' => 0,
        'products_method' => 'any',
        'max_per_cart' => 0,
      ];
    }

    $discount->valid_until = [
      'year' => format_date($discount->valid_until, 'custom', 'Y'),
      'month' => format_date($discount->valid_until, 'custom', 'n'),
      'day' => format_date($discount->valid_until, 'custom', 'j'),
    ];

    $form['main'] = [
      '#type' => 'fieldset',
      '#title' => t('General settings'),
    ];
    $form['main']['name'] = [
      '#type' => 'textfield',
      '#title' => t('Discount name'),
      '#default_value' => $discount->name,
      '#required' => TRUE,
    ];
    $form['main']['valid_until'] = [
      '#type' => 'date',
      '#title' => t('Expiry date'),
      '#default_value' => $discount->valid_until,
      '#required' => TRUE,
    ];
    $form['main']['status'] = [
      '#type' => 'checkbox',
      '#title' => t('Active'),
      '#description' => t('Check to enable the discount, uncheck to disable the discount.'),
      '#default_value' => $discount->status,
    ];
    $form['main']['type'] = [
      '#type' => 'select',
      '#title' => t('Discount type'),
      '#default_value' => $discount->type,
      '#options' => [
        'percentage' => 'Percentage',
        'fixed' => 'Fixed Value',
      ],
      '#description' => t("Note: If you use a percentage and there are products criteria, the percentage will be calculated from the first product's cost."),
    ];
    $form['main']['value'] = [
      '#type' => 'textfield',
      '#title' => t('Discount value'),
      '#default_value' => $discount->value,
      '#size' => 10,
      '#description' => t('Enter values without symbols, for 15%, enter "15" and choose Percentage as the discount type.'),
      '#required' => TRUE,
    ];
    $form['main']['scope'] = [
      '#type' => 'select',
      '#title' => t('Scope'),
      '#required' => TRUE,
      '#default_value' => $discount->scope,
      '#options' => ms_core_get_scopes(),
      '#description' => t('Whether the discount should affect the first payment only or all payments. Only applies for Recurring Payments.'),
    ];
    $form['main']['max_uses'] = [
      '#type' => 'textfield',
      '#title' => t('Maximum number of redemptions'),
      '#default_value' => $discount->max_uses,
      '#description' => t('Enter the maximum number of times this discount can be used, or 0 for unlimited.'),
      '#size' => 5,
      '#required' => TRUE,
    ];
    $form['main']['max_per_user'] = [
      '#type' => 'textfield',
      '#title' => t('Maximum number of uses (per user)'),
      '#default_value' => isset($discount->data['max_per_user']) ? $discount->data['max_per_user'] : 0,
      '#description' => t('Enter the maximum number of times this discount can be used by a single user, or 0 for unlimited.'),
      '#size' => 5,
      '#required' => TRUE,
    ];
    $form['main']['max_per_cart'] = [
      '#type' => 'textfield',
      '#title' => t('Maximum number of uses (per order)'),
      '#default_value' => isset($discount->data['max_per_cart']) ? $discount->data['max_per_cart'] : 0,
      '#description' => t('Enter the maximum number of times this discount can be used in a single order, or 0 for unlimited.'),
      '#size' => 5,
      '#required' => TRUE,
    ];

    $form['criteria'] = [
      '#type' => 'fieldset',
      '#title' => t('Criteria'),
      '#description' => t("All of the criteria set must be met in order for the discount to be applied to a cart."),
    ];
    $form['criteria']['minimum_order'] = [
      '#type' => 'textfield',
      '#title' => t('Minimum order limit'),
      '#default_value' => $discount->data['minimum_order'],
      '#size' => 10,
      '#description' => t('A minimum order total that applies to the discount, or 0 for no minimum order limit.'),
      '#required' => TRUE,
    ];

    $form['criteria']['users'] = [
      '#type' => 'fieldset',
      '#title' => t('User restrictions'),
      '#description' => t('Enter one or more user names and/or "anonymous users" below to make this discount valid only for those users.'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => empty($discount->data['users']),
    ];

    if (isset($discount->data['users'])) {
      foreach ($discount->data['users'] as $username) {
        $form['criteria']['users'][] = [
          '#type' => 'textfield',
          '#default_value' => $username,
          '#autocomplete_path' => 'ms_core/autocomplete/user',
        ];
      }
    }

    for ($i = 0; $i < 4; $i++) {
      $form['criteria']['users'][] = [
        '#type' => 'textfield',
        '#autocomplete_path' => 'ms_core/autocomplete/user',
      ];
    }

    $form['criteria']['roles'] = [
      '#type' => 'fieldset',
      '#title' => t('Role restrictions'),
      '#description' => t('Enter one or more role names below to make this discount valid only for users with those roles.'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => empty($discount->data['roles']),
    ];

    if (isset($discount->data['roles'])) {
      foreach ($discount->data['roles'] as $role) {
        $form['criteria']['roles'][] = [
          '#type' => 'textfield',
          '#default_value' => $role,
          '#autocomplete_path' => 'ms_core/autocomplete/role',
        ];
      }
    }

    for ($i = 0; $i < 4; $i++) {
      $form['criteria']['roles'][] = [
        '#type' => 'textfield',
        '#autocomplete_path' => 'ms_core/autocomplete/role',
      ];
    }

    $form['criteria']['products_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => t('Product restrictions'),
      '#collapsible' => TRUE,
      '#collapsed' => empty($discount->data['products']),
    ];
    $form['criteria']['products_fieldset']['products_method'] = [
      '#type' => 'select',
      '#title' => t('Method'),
      '#options' => [
        'all' => t("All"),
        'any' => t("One or more"),
      ],
      '#description' => t("If 'All' is selected, then all products listed below need to be in the cart for the discount to apply. If 'Any' is selected, then only one of them needs to be present. You may use '*' for wildcards."),
      '#default_value' => $discount->data['products_method'],
    ];
    $form['criteria']['products_fieldset']['products'] = [
      '#type' => 'fieldset',
      '#title' => t('Product restrictions'),
      '#description' => t('Enter one or more products below to make this discount valid only for those products. Just enter the product name and it will be auto-completed.'),
      '#tree' => TRUE,
    ];
    if (isset($discount->data['products'])) {
      foreach ($discount->data['products'] as $id) {
        $form['criteria']['products_fieldset']['products'][] = [
          '#type' => 'textfield',
          '#default_value' => $id,
          '#autocomplete_path' => 'ms_core/autocomplete/product',
        ];
      }
    }
    for ($i = 0; $i < 4; $i++) {
      $form['criteria']['products_fieldset']['products'][] = [
        '#type' => 'textfield',
        '#autocomplete_path' => 'ms_core/autocomplete/product',
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Submit'),
    ];

    return $form;
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $valid_until = mktime(0, 0, 0, $form_state->getValue(['valid_until', 'month']), $form_state->getValue(['valid_until', 'day']), $form_state->getValue(['valid_until', 'year']));
    $data = [];

    $data['max_per_user'] = $form_state->getValue(['max_per_user']);
    $data['minimum_order'] = $form_state->getValue(['minimum_order']);
    $data['products_method'] = $form_state->getValue(['products_method']);
    $data['max_per_cart'] = $form_state->getValue(['max_per_cart']);

    foreach ($form_state->getValue(['users']) as $username) {
      $data['users'][] = $username;
    }

    foreach ($form_state->getValue(['roles']) as $role) {
      if ($role) {
        $data['roles'][] = $role;
      }
    }

    foreach ($form_state->getValue(['products']) as $id) {
      if ($id) {
        $data['products'][] = $id;
      }
    }

    if (!isset($form['#ms_discounts_id'])) {
      db_insert('ms_discounts')
        ->fields([
        'name' => $form_state->getValue(['name']),
        'value' => floatval($form_state->getValue(['value'])),
        'type' => $form_state->getValue(['type']),
        'scope' => $form_state->getValue(['scope']),
        'status' => $form_state->getValue(['status']),
        'valid_until' => $valid_until,
        'max_uses' => $form_state->getValue(['max_uses']),
        'data' => serialize($data),
      ])
        ->execute();
      drupal_set_message(t('Discount %name has been created.', [
        '%name' => $form_state->getValue(['name'])
        ]));
    }
    else {
      db_update('ms_discounts')
        ->fields([
        'name' => $form_state->getValue(['name']),
        'value' => floatval($form_state->getValue(['value'])),
        'type' => $form_state->getValue(['type']),
        'scope' => $form_state->getValue(['scope']),
        'status' => $form_state->getValue(['status']),
        'valid_until' => $valid_until,
        'max_uses' => $form_state->getValue(['max_uses']),
        'data' => serialize($data),
      ])
        ->condition('id', $form['#ms_discounts_id'])
        ->execute();
      drupal_set_message(t('Discount %name has been updated.', [
        '%name' => $form_state->getValue(['name'])
        ]));
    }

    $form_state->set(['redirect'], 'admin/moneysuite/ms_discounts');
  }

}
