<?php

/**
 * @file
 * Contains \Drupal\ms_tax\Form\MsTaxAdmin.
 */

namespace Drupal\ms_tax\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class MsTaxAdmin extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ms_tax_admin';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ms_tax.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ms_tax.settings'];
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $form['main'] = [
      '#type' => 'fieldset',
      '#title' => t('Main Settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    ];

    // @FIXME
    // Could not extract the default value because it is either indeterminate, or
    // not scalar. You'll need to provide a default value in
    // config/install/ms_tax.settings.yml and config/schema/ms_tax.schema.yml.
    $form['main']['ms_tax_name'] = [
      '#type' => 'textfield',
      '#title' => t('Display Text for Tax in Cart and Order'),
      '#size' => 32,
      '#maxlength' => 64,
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_name'),
      '#description' => t("This is what will appear next to the Tax amount on orders."),
      '#required' => TRUE,
      '#ms_override' => TRUE,
    ];
    $form['main']['ms_tax_amount'] = [
      '#type' => 'textfield',
      '#title' => t('Tax Amount to apply to orders by default'),
      '#size' => 32,
      '#maxlength' => 64,
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_amount'),
      '#description' => t("This tax will apply to all orders by default."),
      '#required' => TRUE,
      '#ms_override' => TRUE,
    ];
    $form['main']['ms_tax_type'] = [
      '#type' => 'radios',
      '#title' => t('Percentage or Fixed Amount?'),
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_type'),
      '#options' => [
        'percentage' => t('Percentage'),
        'fixed' => t('Fixed'),
      ],
      '#required' => TRUE,
      '#ms_override' => TRUE,
    ];
    $scopes = ms_core_get_scopes();
    $form['main']['ms_tax_scope'] = [
      '#type' => 'radios',
      '#title' => t('Scope'),
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_scope'),
      '#options' => $scopes,
      '#required' => TRUE,
      '#description' => t('Whether the tax should affect the first payment only or all payments. Only applies for Recurring Payments.'),
      '#ms_override' => TRUE,
    ];
    $form['main']['ms_tax_weight'] = [
      '#type' => 'select',
      '#title' => t('Weight'),
      '#description' => t("Select the weight for the tax adjustment. The weight controls what order adjustments are calculated in."),
      '#options' => array_combine(range(-50, 50), range(-50, 50)),
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_weight'),
    ];

    $form['countries'] = [
      '#type' => 'fieldset',
      '#title' => t('Country and Region Specific Settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#description' => t("Here you can specify country and region specific settings for taxes.
      This can be very useful for charging VAT for different countries at different rates.
      If you have a profile field set in the MS Core Settings for the Billing Country or Region, then that field will be used to determine the default country or region associated with the user.
      If the user is not logged in, then if the profile field is included in the registration form, it will be used to determine the country or region.
      Other means are also used to attempt to automatically determine the country or region based on the user's past information."),
    ];
    $form['countries']['ms_tax_use_country_specific_rates'] = [
      '#type' => 'checkbox',
      '#title' => t('Use Country and/or Region Specific Rates?'),
      '#size' => 32,
      '#maxlength' => 64,
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_use_country_specific_rates'),
      '#description' => t("If this is checked, country and / or region fields will be added to the Checkout form so that users can select which country and region they are in for tax purposes."),
    ];

    $form['countries']['regions'] = [
      '#type' => 'fieldset',
      '#title' => t('Region Specific Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => \Drupal::config('ms_tax.settings')->get('ms_tax_state_specific_rates'),
    ];


    $form['countries']['regions']['ms_tax_state_specific_rates'] = [
      '#type' => 'textarea',
      '#title' => t('Region Specific Settings'),
      '#description' => t("Here you can specify state/province specific settings for taxes.
      This can be very useful for charging VAT for different provinces at different rates.
      If you have a profile field set in the MS Core Settings for the Billing State,
      then that field will be used to determine the state/province associated with the user.
      If the user is not logged in, then if the profile field is included in the registration
      form, it will be used to determine the state/province. If all of these methods fail,
      then the user will be asked to select a state/province before they go
      to checkout.") . "<br />" . t("Enter the state 2-letter codes, tax names, and tax amounts on separate lines, divided by a | character. Ex: TX|Tax for Texas|5.95"),
      '#default_value' => \Drupal::config('ms_tax.settings')->get('ms_tax_state_specific_rates'),
    ];

    $countries_list = ms_core_get_countries('iso');
    // @FIXME
    // Could not extract the default value because it is either indeterminate, or
    // not scalar. You'll need to provide a default value in
    // config/install/ms_tax.settings.yml and config/schema/ms_tax.schema.yml.
    $specific_countries = \Drupal::config('ms_tax.settings')->get('ms_tax_specific_countries');

    if (!$form_state->get(['specific_countries']) AND is_array($form_state->get(['specific_countries']))) {
      $specific_countries = array_merge($specific_countries, $form_state->get(['specific_countries']));
    }

    foreach ($specific_countries as $country_iso) {
      $form['countries']['specific_countries'][$country_iso] = [
        '#type' => 'fieldset',
        '#title' => t('Country Specific Settings for @name', [
          '@name' => $countries_list[$country_iso]
          ]),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      ];

      // @FIXME
      // // @FIXME
      // // The correct configuration object could not be determined. You'll need to
      // // rewrite this call manually.
      // $form['countries']['specific_countries'][$country_iso]['ms_tax_name_' . $country_iso] = array(
      //       '#type' => 'textfield',
      //       '#title' => t('Display Text for Tax in Cart and Order'),
      //       '#size' => 32,
      //       '#maxlength' => 64,
      //       '#default_value' => variable_get('ms_tax_name_' . $country_iso, t('Tax')),
      //       '#description' => t("This is what will appear next to the Tax amount on orders."),
      //       '#required' => TRUE,
      //       '#ms_override' => TRUE,
      //     );

      // @FIXME
      // // @FIXME
      // // The correct configuration object could not be determined. You'll need to
      // // rewrite this call manually.
      // $form['countries']['specific_countries'][$country_iso]['ms_tax_amount_' . $country_iso] = array(
      //       '#type' => 'textfield',
      //       '#title' => t('Tax Amount to apply to orders by default'),
      //       '#size' => 32,
      //       '#maxlength' => 64,
      //       '#default_value' => variable_get('ms_tax_amount_' . $country_iso, 0),
      //       '#description' => t("This tax will apply to all orders by default."),
      //       '#required' => TRUE,
      //       '#ms_override' => TRUE,
      //     );

      // @FIXME
      // // @FIXME
      // // The correct configuration object could not be determined. You'll need to
      // // rewrite this call manually.
      // $form['countries']['specific_countries'][$country_iso]['ms_tax_type_' . $country_iso] = array(
      //       '#type' => 'radios',
      //       '#title' => t('Percentage or Fixed Amount?'),
      //       '#default_value' => variable_get('ms_tax_type_' . $country_iso, 'percentage'),
      //       '#options' => array(
      //         'percentage' => t('Percentage'),
      //         'fixed' => t('Fixed'),
      //       ),
      //       '#required' => TRUE,
      //       '#ms_override' => TRUE,
      //     );

      // @FIXME
      // // @FIXME
      // // The correct configuration object could not be determined. You'll need to
      // // rewrite this call manually.
      // $form['countries']['specific_countries'][$country_iso]['ms_tax_scope_' . $country_iso] = array(
      //       '#type' => 'radios',
      //       '#title' => t('Scope'),
      //       '#default_value' => variable_get('ms_tax_scope_' . $country_iso, 'recurring'),
      //       '#options' => $scopes,
      //       '#required' => TRUE,
      //       '#description' => t('Whether the tax should affect the first payment only or all payments. Only applies for Recurring Payments.'),
      //       '#ms_override' => TRUE,
      //     );

      $form['countries']['specific_countries'][$country_iso]['ms_tax_remove_country-' . $country_iso] = [
        '#type' => 'submit',
        '#value' => t('Remove @name', [
          '@name' => $countries_list[$country_iso]
          ]),
        '#submit' => ['ms_tax_remove_country'],
        // Since we are removing a country, don't validate until later.
      '#limit_validation_errors' => [],
      ];
    }

    $form['countries']['add'] = [
      '#type' => 'fieldset',
      '#title' => t('New Country Specific Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];
    $form['countries']['add']['ms_tax_new_country'] = [
      '#type' => 'select',
      '#title' => t('Country'),
      '#options' => $countries_list,
      '#required' => FALSE,
    ];
    $form['countries']['add']['ms_tax_add_country'] = [
      '#type' => 'submit',
      '#value' => t('Add Country Specific Settings'),
      '#submit' => [
        'ms_tax_add_country'
        ],
    ];

    return parent::buildForm($form, $form_state);
  }

}
