<?php /**
 * @file
 * Contains \Drupal\ms_events\Controller\DefaultController.
 */

namespace Drupal\ms_events\Controller;

use Drupal\Core\Controller\ControllerBase;

/**
 * Default controller for the ms_events module.
 */
class DefaultController extends ControllerBase {

  public function ms_events_view_purchased_tickets_access_test(Drupal\Core\Session\AccountInterface $account) {
    $user = \Drupal::currentUser();
    return (\Drupal::currentUser()->hasPermission('administer users') OR (\Drupal::currentUser()->hasPermission('view own purchased tickets') && ($account->id() == $user->uid)));
  }

  public function ms_events_user_purchased_tickets_page(\Drupal\user\UserInterface $account) {
    $output = '';
    $headers = [t('Title'), t('Seat'), t('Date'), t('Actions')];
    $result = db_query("SELECT * FROM {ms_events_tickets} WHERE uid = :uid", [
      ':uid' => $account->id()
      ]);
    $rows = [];
    $attr = [];
    foreach ($result as $row) {
      $actions = [];
      // @FIXME
      // l() expects a Url object, created from a route name or external URI.
      // $actions[] = l(t('View Ticket'), 'user/' . $account->uid . '/events_tickets/' . $row->id);

      if ($row->oid AND $order = ms_core_order_load($row->oid)) {
        // @FIXME
// l() expects a Url object, created from a route name or external URI.
// $actions[] = l(t('View Order'), 'user/' . $account->uid . '/order-history/view/' . $order->order_key);

      }
      $entity = _ms_events_entity_load_single($row->entity_type, $row->entity_id);
      $uri = entity_uri($row->entity_type, $entity);
      $label = $entity->label();
      $ticket_info = ms_events_extract_ticket_info($entity, $row->field_name, $row->eid);

      // @FIXME
      // l() expects a Url object, created from a route name or external URI.
      // $rows[] = array(
      //       ($uri) ? l($label, $uri['path']) : $label,
      //       $row->ticket_seat ? $row->ticket_seat : t('N/A'),
      //       format_date($ticket_info['event_date'], 'short'),
      //       implode($actions, ' | '),
      //     );

    }

    // @FIXME
    // theme() has been renamed to _theme() and should NEVER be called directly.
    // Calling _theme() directly can alter the expected output and potentially
    // introduce security issues (see https://www.drupal.org/node/2195739). You
    // should use renderable arrays instead.
    // 
    // 
    // @see https://www.drupal.org/node/2195739
    // $output .= theme('table', array('header' => $headers, 'rows' => $rows, 'attributes' => $attr, 'empty' => t('No tickets.')));

    return $output;
  }

  public function ms_events_user_view_ticket_page(\Drupal\user\UserInterface $account, $ticket) {
    $entity = _ms_events_entity_load_single($ticket->entity_type, $ticket->entity_id);
    $uri = entity_uri($ticket->entity_type, $entity);
    $label = $entity->label();
    $ticket_info = ms_events_extract_ticket_info($entity, $ticket->field_name, $ticket->eid);
    // @FIXME
    // theme() has been renamed to _theme() and should NEVER be called directly.
    // Calling _theme() directly can alter the expected output and potentially
    // introduce security issues (see https://www.drupal.org/node/2195739). You
    // should use renderable arrays instead.
    // 
    // 
    // @see https://www.drupal.org/node/2195739
    // return theme('ms_event_ticket', array(
    //     'ticket' => $ticket,
    //     'account' => $account,
    //     'event_name' => $label,
    //     'event_date' => $ticket_info['event_date'],
    //     'order' => ms_core_order_load($ticket->oid),
    //   ));

  }

  public function ms_events_view_ticket_types() {
    $user = \Drupal::currentUser();
    $build = [];

    $headers = [
      [
        'data' => t('Name'),
        'class' => ['ticket_type_name_header'],
        'field' => 't.name',
        'sort' => 'desc',
      ],
      [
        'data' => t('Type'),
        'class' => ['ticket_type_type_header'],
        'field' => 't.type',
        'sort' => 'desc',
      ],
      ['data' => t('Actions'), 'class' => ['ticket type_actions_header']],
    ];

    $query = db_select('ms_events_ticket_types', 't');
    $query->fields('t');
    $query = $query->extend('TableSort')->orderByHeader($headers);

    $pager_amount = 20;
    if (!isset($_GET['pager_amount']) OR $pager_amount = $_GET['pager_amount']) {
      $query = $query->extend('PagerDefault')->limit($pager_amount);
    }

    // execute the query
    $result = $query->execute();

    $rows = [];
    $attr = ['id' => 'ticket_types-table'];
    foreach ($result as $row) {
      // @FIXME
// l() expects a Url object, created from a route name or external URI.
// $actions = array(
//       l(t('Edit'), 'admin/moneysuite/ms_events/ticket_types/edit/' . $row->type),
//       l(t('Delete'), 'admin/moneysuite/ms_events/ticket_types/delete/' . $row->type),
//     );


      $rows[] = [
        'data' => [
          [
            'data' => $row->name,
            'class' => [
              'ticket_type_name'
              ],
          ],
          ['data' => $row->type, 'class' => ['ticket_type_type']],
          [
            'data' => implode(' | ', $actions),
            'class' => [
              'ticket type_actions'
              ],
          ],
        ]
        ];
    }

    // add the content
    // @FIXME
    // theme() has been renamed to _theme() and should NEVER be called directly.
    // Calling _theme() directly can alter the expected output and potentially
    // introduce security issues (see https://www.drupal.org/node/2195739). You
    // should use renderable arrays instead.
    // 
    // 
    // @see https://www.drupal.org/node/2195739
    // $build['content'] = array(
    //     'table' => array(
    //       '#type' => 'item',
    //       '#markup' => theme('table', array('header' => $headers, 'rows' => $rows, 'attributes' => $attr, 'empty' => t('No ticket types.'))),
    //     ),
    //   );


    // Add the pager
    $build['pager'] = [
      '#theme' => 'pager',
      '#weight' => 5,
    ];

    return $build;
  }

  public function ms_events_autocomplete_event($title = '') {
    $matches = [];
    if ($title) {
      // @FIXME
// Fields and field instances are now exportable configuration entities, and
// the Field Info API has been removed.
// 
// 
// @see https://www.drupal.org/node/2012896
// $fields = field_info_fields();

      $field_options = [];
      $entity_options = [];

      $field_instances = ms_events_get_field_instances();

      foreach ($field_instances as $field) {
        foreach ($field['bundles'] as $entity_type => $bundle) {
          $entities = // @FIXME
            // To reset the entity cache, use EntityStorageInterface::resetCache().
\Drupal::entityManager()->getStorage($entity_type)->loadByProperties([
            'type' => $bundle
            ]);
          foreach ($entities as $entity) {
            $label = $entity->label();
            if (stristr($label, $title)) {
              list($entity_id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);
              $matches[ms_events_create_autocomplete_key($entity_type, $entity_id)] = $label;
            }
          }
        }
      }
    }
    drupal_json_output($matches);
  }

}
