<?php

/**
 * @file
 * Contains \Drupal\ms_events\Form\MsEventsAdmin.
 */

namespace Drupal\ms_events\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class MsEventsAdmin extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ms_events_admin';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ms_events.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ms_events.settings'];
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $form['main'] = [
      '#type' => 'fieldset',
      '#title' => t('Main Settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#description' => t("MS Events Settings"),
    ];

    $form['main']['ms_events_ticket_cart_hold_time'] = [
      '#type' => 'select',
      '#title' => t('Ticket Hold Time'),
      '#options' => [
        '5 minutes' => t('5 minutes'),
        '10 minutes' => t('10 minutes'),
        '15 minutes' => t('15 minutes'),
        '20 minutes' => t('20 minutes'),
        '30 minutes' => t('30 minutes'),
      ],
      '#default_value' => \Drupal::config('ms_events.settings')->get('ms_events_ticket_cart_hold_time'),
      '#description' => t("Choose how long will the tickets be reserved while they
      are in someone's cart. Once the time runs out, the tickets will be removed
      from the cart and available for purchase by someone else."),
      '#required' => TRUE,
    ];
    $form['main']['ms_events_max_quantity'] = [
      '#type' => 'textfield',
      '#title' => t('Max ticket quantity per purchase'),
      '#default_value' => \Drupal::config('ms_events.settings')->get('ms_events_max_quantity'),
      '#description' => t("Set the max number of tickets that can be purchased at one time. This can
      help prevent users from adding all tickets to their cart, preventing others
      from purchasing the tickets."),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

}
