<?php

/**
 * @file
 * Contains \Drupal\ms_events\Form\MsEventsEditTicketForm.
 */

namespace Drupal\ms_events\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class MsEventsEditTicketForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ms_events_edit_ticket_form';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state, $id = NULL) {
    $is_edit = FALSE;
    if ($id != NULL) {
      if ($record = ms_events_get_ticket_by_id($id)) {
        $is_edit = TRUE;
        $form['id'] = [
          '#type' => 'value',
          '#value' => $record->id,
        ];
        $form_state->setValue(['event'], ms_events_create_autocomplete_key($record->entity_type, $record->entity_id));
        $account = \Drupal::entityManager()->getStorage('user')->load($record->uid);
      }
    }
    $form['username'] = [
      '#type' => 'textfield',
      '#title' => t('User'),
      '#description' => t("The User this ticket is for"),
      '#autocomplete_path' => 'ms_core/autocomplete/user',
      '#required' => TRUE,
      '#default_value' => ($is_edit) ? $account->name : '',
    ];
    $form['event'] = [
      '#type' => 'textfield',
      '#title' => t('Event'),
      '#description' => t("Enter the name of the event"),
      '#autocomplete_path' => 'ms_events/autocomplete/event',
      '#required' => TRUE,
      '#default_value' => ($is_edit) ? ms_events_create_autocomplete_key($record->entity_type, $record->entity_id) : '',
      '#ajax' => [
        'callback' => 'ms_events_tickets_form_ajax_callback',
        'wrapper' => 'ticket_info-div',
        'method' => 'replace',
        'effect' => 'fade',
      ],
    ];
    // @FIXME
    // Fields and field instances are now exportable configuration entities, and
    // the Field Info API has been removed.
    // 
    // 
    // @see https://www.drupal.org/node/2012896
    // $fields = field_info_fields();


    $field_types = ms_events_get_fields();
    $field_type_map = [];

    $field_options = [];
    if ($form_state->getValue(['event'])) {
      if ($entity_info = ms_events_extract_from_autocomplete_key($form_state->getValue([
        'event'
        ]))) {
        extract($entity_info);
        $entity = _ms_events_entity_load_single($entity_type, $entity_id);
        foreach ($fields as $field) {
          if (isset($field_types[$field['type']]) AND isset($entity->{$field['field_name']}['und'])) {
            foreach ($entity->{$field['field_name']}['und'] as $field_info) {
              $field_type_map[$field['field_name']] = $field['type'];
              $field_options[$field['field_name'] . '|' . $field_info['eid']] = $field['field_name'] . ' ' . format_date($field_info['event_date']);
            }
          }
        }
      }
    }

    $form['ticket_info'] = [
      '#type' => 'fieldset',
      '#title' => t('Ticket Info'),
      '#prefix' => '<div id="ticket_info-div">',
      '#suffix' => '</div>',
    ];

    $form['ticket_info']['field_eid'] = [
      '#type' => 'select',
      '#title' => t('Field'),
      '#options' => $field_options,
      '#description' => t("Choose the field."),
      '#required' => FALSE,
      '#ajax' => [
        'callback' => 'ms_events_tickets_form_ajax_callback',
        'wrapper' => 'ticket_info-div',
        'method' => 'replace',
        'effect' => 'fade',
      ],
      '#default_value' => ($is_edit) ? $record->field_name . '|' . $record->eid : '',
    ];

    if (!$form_state->getValue(['field_eid']) AND !empty($field_options)) {
      $form_state->setValue(['field_eid'], key($field_options));
    }

    if ($form_state->getValue(['field_eid'])) {
      list($field_eid_name, $field_eid_eid) = explode('|', $form_state->getValue([
        'field_eid'
        ]));
      $field_eid_type = $field_type_map[$field_eid_name];
      $seat_options = $field_types[$field_eid_type]['get_seats']($entity_type, $entity_id, $entity, $field_eid_name, $field_eid_eid);
      if ($is_edit) {
        $seat_options[$record->data['seat_id']] = $record->ticket_seat;
      }
      $form['seat_options'] = [
        '#type' => 'value',
        '#value' => $seat_options,
      ];
      $form['ticket_info']['ticket_seat'] = [
        '#type' => 'select',
        '#title' => t('Seat'),
        '#options' => $seat_options,
        '#default_value' => ($is_edit) ? $record->data['seat_id'] : '',
      ];

      $form['ticket_info']['price'] = [
        '#type' => 'textfield',
        '#title' => t('Price'),
        '#description' => t("Enter the price of this ticket"),
        '#default_value' => ($is_edit) ? $record->price : '',
      ];
    }

    $form['status'] = [
      '#type' => 'select',
      '#title' => t('Status'),
      '#options' => ms_events_get_statuses(),
      '#description' => t("What is the Status of this Ticket?"),
      '#required' => FALSE,
      '#default_value' => $is_edit ? $record->status : 1,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Save'),
    ];
    $form_state->set(['redirect'], 'admin/ms_events_tickets');
    return $form;
  }

  public function validateForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $v = $form_state->getValues();

    if (!$account = user_load_by_name($v['username'])) {
      $form_state->setErrorByName('username', t('Error: Please enter a valid user.'));
    }
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $v = $form_state->getValues();
    list($field_name, $eid) = explode('|', $v['field_eid']);
    $entity_info = ms_events_extract_from_autocomplete_key($v['event']);
    extract($entity_info);
    $entity = _ms_events_entity_load_single($entity_type, $entity_id);

    if (!isset($v['id']) || !$record = ms_events_get_ticket_by_id($v['id'])) {
      $record = new stdClass();
    }

    $account = user_load_by_name($v['username']);
    $record->entity_id = $entity_id;
    $record->entity_type = $entity_type;
    $record->field_name = $field_name;
    $record->eid = $eid;
    $record->uid = $account->uid;
    $record->status = $v['status'];
    $record->price = $v['price'];
    $record->ticket_seat = $v['seat_options'][$v['ticket_seat']];
    $record->data['seat_id'] = $v['ticket_seat'];

    if (!empty($record->id)) {
      \Drupal::database()->merge('ms_events_tickets')->fields($record)->key([
        'id'
        ])->execute();
    }
    else {
      \Drupal::database()->insert('ms_events_tickets')->fields($record)->execute();
    }

    $title = $entity->label();
    drupal_set_message(t("Ticket to @event for @user has been saved.", [
      '@user' => $account->name,
      '@event' => $title,
    ]));
    $form_state->set(['redirect'], 'admin/ms_events_tickets');
  }

}
