<?php

/**
 * @file
 * Contains \Drupal\ms_events\Form\MsEventsEditTicketTypeForm.
 */

namespace Drupal\ms_events\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class MsEventsEditTicketTypeForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ms_events_edit_ticket_type_form';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state, $type = NULL) {
    $is_edit = FALSE;
    if ($type != NULL) {
      if ($record = ms_events_get_ticket_type($type)) {
        $is_edit = TRUE;

      }
    }
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#required' => TRUE,
      '#description' => t("Enter the ticket type name."),
      '#default_value' => $is_edit ? $record->name : '',
    ];
    if ($is_edit) {
      $form['type'] = [
        '#type' => 'value',
        '#value' => $record->type,
      ];
    }
    else {
      $form['type'] = [
        '#type' => 'machine_name',
        '#title' => t('Title'),
        '#required' => TRUE,
        '#machine_name' => [
          'exists' => 'ms_events_get_ticket_type'
          ],
        '#default_value' => $is_edit ? $record->type : '',
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Save'),
    ];
    $form_state->set(['redirect'], 'admin/moneysuite/ms_events/ticket_types');
    return $form;
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $v = $form_state->getValues();

    if ($record = ms_events_get_ticket_type($v['type'])) {
      // We are editing an existing record
      $record->name = $v['name'];
      \Drupal::database()->merge('ms_events_ticket_types')->fields($record)->key([
        'type'
        ])->execute();
    }
    else {
      $record = new stdClass();
      $record->type = $v['type'];
      $record->name = $v['name'];
      \Drupal::database()->insert('ms_events_ticket_types')->fields($record)->execute();
    }

    drupal_set_message(t("Ticket type @title has been saved.", [
      '@title' => $v['name']
      ]));
    $form_state->set(['redirect'], 'admin/moneysuite/ms_events/ticket_types');
  }

}
